#version 330 core

in vec2 texcoords;

uniform sampler2D position_texture;
uniform sampler2D normal_texture;
uniform sampler2D noise_texture;

uniform mat4 viewMatrix;
uniform mat4 projectionMatrix;

const int maxNumSamples = 16;
uniform int numSamples;
uniform vec3 samples[maxNumSamples];

uniform float radius = 0.5;
uniform vec2 noiseScale = vec2(1024.0/4.0, 768.0/4.0);

layout(location = 0) out vec4 fragColor;


// http://john-chapman-graphics.blogspot.co.at/2013/01/ssao-tutorial.html
void main()
{
   // Position of the Fragment in View Space
    vec3 position = (viewMatrix * vec4(texture(position_texture, texcoords).xyz, 1)).xyz;

   // Position of the normal in View Space
   vec3 normal = mat3(viewMatrix) * texture(normal_texture, texcoords).xyz;
   normal = normalize(normal);

   // Construct change-of-basis matrix to reorient our sample kernel along the origin's normal.
   vec3 randomVec = texture(noise_texture, texcoords * noiseScale).xyz * 2.0 - 1.0;
   vec3 tangent = normalize(randomVec - normal * dot(randomVec, normal));
   vec3 bitangent = cross(normal, tangent);
   mat3 tbnMatrix = mat3(tangent, bitangent, normal);

   float occlusion = 0;

   for(int i = 0; i < min(numSamples, maxNumSamples); i++)
   {
		// Get Sample Position:
		vec3 samplePos = tbnMatrix * samples[i];	// From Tangent to view Space
		samplePos = position + samplePos * radius;

		// Project Sample Position:
		vec4 offset = vec4(samplePos, 1.0);
		offset = projectionMatrix * offset;	// From View to Clip-Space
		offset.xyz /= offset.w;				// Perspective Divide
		offset.xy = offset.xy * 0.5 + 0.5;	// Transform to Range 0.0 -1.0

		 // Get Sample Depth: 		
		 float sampleDepth =  (viewMatrix * vec4(texture(position_texture, offset.xy).xyz, 1)).z;

		// Range Check & Accumulate: 
		 float rangeCheck = abs(position.z - sampleDepth) < radius ? 1.0 : 0.0;
		 occlusion +=(sampleDepth >= samplePos.z ? 1.0 : 0.0) * rangeCheck;	
   }

	occlusion = 1 - (occlusion / float(numSamples));
	fragColor = vec4(occlusion,occlusion, occlusion, 1);
}